import os
from .toolkit import _load_from_env

_load_from_env(mode="da")
_load_from_env(mode="ds")


def main():
    loc, da, ds = location()
    print(f"File location is '{loc}' with folders {da} and {ds} for methods")


def location():
    loc = os.path.abspath(__file__)
    directory = os.path.dirname(loc)
    da = os.path.join(directory, "dataarray_methods")
    ds = os.path.join(directory, "dataset_methods")
    return (loc, da, ds)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding=utf-8
"""Extensions for Xarray for analysing weather data."""
import functools
import importlib as ilib
import os
import sys
import xarray as xr
import logging

_DA_ENV = "XSUITE_DA_METHODS"
_DS_ENV = "XSUITE_DS_METHODS"

logger = logging.getLogger(__name__)


@xr.register_dataarray_accessor("imktk")
class _NyxDA(object):
    def __init__(self, data):
        self._data = data

    def _add_methods(self, folder):
        return _add_folder(folder, mode="da")

    def _load_env(self):
        return _load_from_env(mode="da")


@xr.register_dataset_accessor("imktk")
class _NyxDS(object):
    def __init__(self, data):
        self._data = data

    def _add_methods(self, folder):
        return _add_folder(folder, mode="ds")

    def _load_env(self):
        return _load_from_env(mode="ds")


def _patch(func, funcname, mode):
    classobject = _NyxDA if mode == "da" else _NyxDS

    @functools.wraps(func)
    def method(accessor, *args, **kwargs):
        return func(accessor._data, *args, **kwargs)

    setattr(classobject, funcname, method)
    return func


def xtend_dataarray(da_folder=None):
    return _add_folder(da_folder, "da")


def xtend_dataset(ds_folder=None):
    return _add_folder(ds_folder, "ds")


def _add_folder(folder, mode=None):
    if not isinstance(mode, str) or mode.lower() not in ["ds", "da"]:
        raise ValueError("Can not understand mode {}".format(mode))
    mode = mode.lower()
    env = "XTEND_DS" if mode == "ds" else "XTEND_DA"

    if not folder:
        folder = os.environ.get(env, False)
    if not folder:
        return False
    assert os.path.isdir(folder), '"{}" is not a folder'.format(folder)
    folder = os.path.realpath(folder)

    pythonfiles = [x[:-3] for x in os.listdir(folder) if x.endswith(".py") and x[0] != "_"]
    if not pythonfiles:
        return False
    sys.path.insert(0, folder)
    for method in pythonfiles:
        try:
            lib = ilib.import_module(method, package="imktk")
            _patch(getattr(lib, "main"), method, mode)
        except (SystemError, ImportError) as err:
            logger.info("Method: %s not loaded. Because: %s", method, err)
        except AttributeError as err:
            logger.info("Method: %s has no 'main' function (err: %s).", method, err)
    return True


def _load_from_env(mode=None):
    if mode is None:
        _ = [_load_from_env(x) for x in ["da", "ds"]]
        return None
    envvar = _DA_ENV if mode == "da" else _DS_ENV
    env = os.environ.get(envvar, False)
    if not env:
        return None
    folders = env.split(":")
    for folder in folders:
        logger.info("Adding folder %s as %s (ENV: %s)", folder, mode, env)
        _add_folder(folder, mode)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['imktk', 'imktk.dataarray_methods']

package_data = \
{'': ['*'], 'imktk': ['dataset_methods/*']}

install_requires = \
['netCDF4>=1.5.8,<2.0.0', 'xarray>=0.20.1,<0.21.0']

entry_points = \
{'console_scripts': ['imktk = imktk:main']}

setup_kwargs = {
    'name': 'imktk',
    'version': '0.1.4',
    'description': 'Toolkit provided by IMK at KIT',
    'long_description': None,
    'author': 'Uğur Çayoğlu',
    'author_email': 'cayoglu@me.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
